/**
Una scacchiera 3x3 per il gioco Tris.
*/

public class Tris {
	private char[][] board;
	private static final int ROWS = 3;
	private static final int COLUMNS = 3;
	private TrisPlayer p1,p2,currentplayer,winner;
	private TrisIA IA;

/**
Costruisce una scacchiera vuota.
*/
public Tris(boolean p1IA, boolean p2IA) {
	iniziaPartita(p1IA,p2IA);
}

private void iniziaPartita(boolean p1IA, boolean p2IA) {
	board = new char[ROWS][COLUMNS];
		// 	riempi di spazi
	for (int i = 0; i < ROWS; i++)
		for (int j = 0; j < COLUMNS; j++)
			board[i][j] = ' ';
	p1 = new TrisPlayer(p1IA);
	p2 = new TrisPlayer(p2IA);
	currentplayer = p1;
	winner = null;
	IA = new TrisIA(this);
}

public void setPlayerControl(int player, boolean IA) {
	if (player==1) p1.setIA(IA);
	else if (player==2) p2.setIA(IA);
}

public void restart(boolean p1IA, boolean p2IA) {
	iniziaPartita(p1IA,p2IA);
}
public boolean mossaPossibile(int i, int j) {
	return((!partitaFinita()) && (i>=0 && j>=0 && i<3 && j<3) && (board[i][j] == ' '));
}
public boolean setCasella(int i, int j) {
	if (mossaPossibile(i,j)){
		board[i][j] = currentplayer.getSimbolo();
		//System.out.println("setCasella chiamato: i="+i+" j="+j);
		if (partitaVinta()) setVincitore(currentplayer);
		changeplayer();
		return true;
	} else
		return false;
}
public boolean setCasella(TrisMossa m) {
	return setCasella(m.getRow(),m.getCol());
}
public void unsetCasella(int i, int j) {
	changeplayer();
	board[i][j] = ' ';
}
public void unsetCasella(TrisMossa m){
	unsetCasella(m.getRow(),m.getCol());
}
public void doComputerMove() {
	boolean var;
	do {
		 var = setCasella(IA.getMossaMigliore());
	} while (!var);
}

public boolean isIcs(int i, int j) {
	return (board[i][j]==p1.getSimbolo());
}
public boolean isCircle(int i, int j) {
	return (board[i][j]==p2.getSimbolo());
}
private void changeplayer() {
	if (currentplayer == p1) currentplayer = p2;
	else currentplayer = p1;
}
public TrisPlayer getCurrentplayer() {
	return currentplayer;
}
public boolean isComputerTurn() {
	return currentplayer.isIA();
}
public boolean isHumanTurn() {
	return (!currentplayer.isIA());
}
public TrisPlayer getVincitore() {
	return winner;
}
private void setVincitore(TrisPlayer player) {
	winner = player;
}
public boolean partitaVinta() {
	char x =' ';
	boolean vinto = false;
	//check righe
	for (int i=0;i<ROWS;i++)
		if ((' '!=(x=board[i][0])) && ((x==board[i][1]) && (x==board[i][2])))
			vinto = true;
	//check colonne
	if (!vinto)
	  for (int i=0;i<COLUMNS;i++)
		if ((' '!=(x=board[0][i])) && ((x==board[1][i]) && (x==board[2][i])))
			vinto = true;
	//check diagonali
	if ((!vinto)&&(' '!=(x=board[0][0])) && ((x==board[1][1]) && (x==board[2][2])))
		vinto = true;
	if ((!vinto)&&(' '!=(x=board[0][2])) && ((x==board[1][1]) && (x==board[2][0])))
		vinto = true;
	return vinto;
}

public boolean partitaPareggiata(){
	for (int i = 0; i < ROWS; i++)
		for (int j = 0; j < COLUMNS; j++)
			if(board[i][j]==' ') return false; // nessuno ha vinto e c'e' spazio
	return (!partitaVinta()); // tutte le caselle occupate, se nessuno ha vinto e' finita pari
}

public boolean partitaFinita() {
	return (partitaVinta() || partitaPareggiata());
}
/**
Crea una rappresentazione della scacchiera
in una stringa, come ad esempio
|x o|
| x |
| o |
@return la stringa rappresentativa
*/
public String toString() {
	String r = "";
	for (int i = 0; i < ROWS; i++) {
		r = r + "|";
		for (int j = 0; j < COLUMNS; j++)
			r = r + board[i][j];
		r = r + "|\n";
	}
	return r;
}
}
