/**
 * @author Dusk - Dario Scarpa - http://www.duskzone.it
 *
 * classe java per la creazione di un pannello da utilizzare
 * come interfaccia grafica per la classe Tris
 * Due listeners si occupano di attendere un click su una casella libera
 * se e' il turno di un giocatore umano o l'elaborazione di una mossa
 * efficace se e' il turno di un giocatore controllato dal computer.
 * Eseguita una mossa, il pannello viene ridisegnato da paintComponent()
 * Una finestra di dialogo mostra infine l'esito della partita
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;

public class TrisBoard extends JPanel {
	private static final Color icscolor = Color.RED;
	private static final Color circlecolor = Color.BLUE;
	private static final Color boardcolor = Color.GRAY;
	private int dimBoard;
	private int dimCasella;
	private Tris game;
	private TrisMouseListener humanListener = new TrisMouseListener();
	private TrisMossaListener iaListener = new TrisMossaListener();
	private Timer IAtimer = new Timer(100, iaListener);
	public TrisBoard(int dim, Tris agame) {
		dimBoard = dim;
		dimCasella = dim/3;
		game = agame;
		startListeners();
	}

	public void reset() {
		repaint();
		startListeners();
	}
	private void startListeners(){
		addMouseListener(humanListener);
		IAtimer.start();
	}
	private void stopListeners(){
		removeMouseListener(humanListener);
		IAtimer.stop();
	}

	private void checkFinePartita() {
		//System.out.println("called checkFinePartita");
		if (game.partitaFinita()) {
			if (game.partitaPareggiata()) {
				JOptionPane.showMessageDialog(null,
	            "partita terminata in pareggio", "PAREGGIO!",
	            JOptionPane.INFORMATION_MESSAGE);
			} else {
				JOptionPane.showMessageDialog(null,
		            "vincitore: "+ game.getVincitore(), "VITTORIA!",
		            JOptionPane.INFORMATION_MESSAGE);
			}
		stopListeners();
		//	System.exit(0);
		}
	}
    private class TrisMossaListener implements ActionListener  {
        public void actionPerformed(ActionEvent event) {
        	stopListeners();
			if (game.isComputerTurn()) {
				game.doComputerMove();
				repaint();
	        	checkFinePartita();
			}
			if (!game.partitaFinita()) startListeners();
        }
    }
	private class TrisMouseListener extends MouseAdapter {
		public void mouseClicked(MouseEvent event) {
		   	stopListeners();
			if (game.isHumanTurn()) {
			  	int x = event.getX(); int y = event.getY();
				int row = -1; int col = -1;
				col = x/dimCasella;
				row = y/dimCasella;
				game.setCasella(col,row);
				repaint();
				checkFinePartita();
			}
			if (!game.partitaFinita()) startListeners();
		}
	}
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D)g;
		BasicStroke mystroke = new BasicStroke(1);
		BasicStroke symbolstroke = new BasicStroke(3);
		Rectangle casella = new Rectangle(dimCasella,dimCasella);
		for (int i=0;i<3;i++)
			for (int j=0;j<3;j++) {
				casella.setLocation(i*dimCasella,j*dimCasella);
				g2.setPaint(new GradientPaint(i*dimCasella,j*dimCasella,Color.white,i*dimCasella+dimCasella,j*dimCasella+dimCasella,boardcolor));
				//g2.setPaint(grad);
				g2.fill(casella);
				g2.setStroke(mystroke);
				g2.setColor(Color.BLACK);
				g2.draw(casella);
				g2.setStroke(symbolstroke);
				if (game.isIcs(i,j)) {
					g2.setColor(icscolor);
					paintIcs(g2,casella.getCenterX(),casella.getCenterY());
				} else if (game.isCircle(i,j)) {
					g2.setColor(circlecolor);
					paintCircle(g2,casella.getCenterX(),casella.getCenterY());
				}
			}
	}
	private void paintIcs(Graphics2D g2,double x, double y) {
		int x_dim = dimCasella - 10;
		Point2D.Double start = new Point2D.Double(x-x_dim/2,y-x_dim/2);
	    Point2D.Double end = new Point2D.Double(x+x_dim/2,y+x_dim/2);
	    Line2D.Double linea = new Line2D.Double(start,end);
	    g2.draw(linea);
	    start.setLocation(x-x_dim/2,y+x_dim/2);
	    end.setLocation(x+x_dim/2,y-x_dim/2);
	    linea.setLine(start,end);
	    g2.draw(linea);
	}
	private void paintCircle(Graphics2D g2,double x, double y) {
		Ellipse2D.Double c = new Ellipse2D.Double();
		int c_wid = dimCasella - 10;
		int c_hei = dimCasella - 10;
		c.height = c_wid;
		c.width = c_hei;
		c.x = x-(c_wid)/2;
		c.y = y-(c_hei)/2;
		g2.draw(c);
	}
}