/**
 * @author Dusk - Dario Scarpa - http://www.duskzone.it
 *
 * classe java per gestire un giocatore controllato dal computer
 * per la classe Tris.
 * Basata sugli algoritmi di esempio per il minimax del libro
 * "Programming Abstractions in C" (Eric Roberts)
 * (capitolo "Backtracking and games"  )
 */

import java.util.ArrayList;

public class TrisIA {
//	*******************************************************
	private class TrisMossaIA extends TrisMossa{
		int rating;
		public TrisMossaIA() {
			super();
		}
		public TrisMossaIA(int i, int j) {
			super(i,j);			
		}
		public void setRating(int aRating){
			rating = aRating;
		}
		public int getRating(){
			return rating;
		}
	}
	//*******************************************************
	public TrisIA(Tris aTris) {
		game = aTris;
		outputmossa = new TrisMossaIA();
	}
	public TrisMossa getMossaMigliore() {
		trovaMossaMigliore(0,outputmossa);
		return (TrisMossa) outputmossa;
	}
	private TrisMossaIA trovaMossaMigliore(int profondita, TrisMossaIA m){
		TrisMossaIA move, bestMove;
		ArrayList moveArrayList = generaListaMosse();
		bestMove = new TrisMossaIA();
		bestMove.setRating(WINNINGPOS+1);
		for (int i=0;i<moveArrayList.size() && bestMove.getRating() != LOSINGPOS ;i++) {
			move = (TrisMossaIA) moveArrayList.get(i);
			game.setCasella((TrisMossa)move);
			move.setRating(ValutaPosizione(profondita+1));
			if (move.getRating()<bestMove.getRating()) {
					bestMove = move;
			}
			game.unsetCasella((TrisMossa)move);
		}
		m.setRow(bestMove.getRow()); m.setCol(bestMove.getCol());
		m.setRating(-bestMove.getRating()); //la peggior mossa per l'avversario e' la migliore da giocare
		return(bestMove);
	}
	private ArrayList generaListaMosse(){
		int cont = 0;
		ArrayList arraymosse= new ArrayList(9);
		for (int i=0; i<3; i++)
			for (int j=0; j<3; j++)
				if (game.mossaPossibile(i,j)) arraymosse.add(new TrisMossaIA(i,j));
		return arraymosse;
	}
	private int ValutaPosizione(int profondita) {
		TrisMossaIA m = new TrisMossaIA();
		if (game.partitaFinita() || profondita > MAXDEPTH) {
			return (ValutaPosizioneStatica());
		}
		trovaMossaMigliore(profondita, m);
		return m.getRating();
	}
	private int ValutaPosizioneStatica() {
		if (game.partitaVinta() && (game.getVincitore()==game.getCurrentplayer())) return WINNINGPOS;
		if (game.partitaVinta()) return LOSINGPOS;
		return NEUTRALPOS;
	}
	
	private TrisMossaIA outputmossa = new TrisMossaIA();
	private Tris game;
	private static final int WINNINGPOS = 1000;
	private static final int NEUTRALPOS = 0;
	private static final int LOSINGPOS = - WINNINGPOS;
	private static final int MAXDEPTH = 10000;
}
